using System;
using System.Configuration;
using System.IO;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using System.Web;
using System.Web.Http;
using System.Web.Http.Cors;
using System.Web.Hosting;
using System.Linq; // 添加Linq引用

namespace ImageUploadApi.Controllers
{
    [EnableCors(origins: "*", headers: "*", methods: "*")]
    public class ImageUploadController : ApiController
    {

        //因为上传的图像数据比较大，请务必在web.config中配置最大请求体大小，否则有可能会造成上次失败
        [HttpPost]
        [Route("api/ImageUpload/upload")]
        public async Task<IHttpActionResult> UploadImage()
        {
            try
            {
                // 增加调试输出，记录请求开始
                System.Diagnostics.Debug.WriteLine("开始处理上传请求...");
                
                if (!Request.Content.IsMimeMultipartContent())
                {
                    System.Diagnostics.Debug.WriteLine("错误：不支持的媒体类型");
                    return BadRequest("不支持的媒体类型");
                }

                var provider = new MultipartFormDataStreamProvider(Path.GetTempPath());
                await Request.Content.ReadAsMultipartAsync(provider);
                
                System.Diagnostics.Debug.WriteLine("已读取请求内容");

                // 获取表单数据
                string imageCount = Array.IndexOf(provider.FormData.AllKeys, "imageCount") >= 0 ? provider.FormData["imageCount"] : "未提供";
                string id = Array.IndexOf(provider.FormData.AllKeys, "id") >= 0 ? HttpUtility.UrlDecode(provider.FormData["id"]) : "未知ID";
                string desc = Array.IndexOf(provider.FormData.AllKeys, "desc") >= 0 ? HttpUtility.UrlDecode(provider.FormData["desc"]) : "未提供描述";
                
                System.Diagnostics.Debug.WriteLine($"表单数据: imageCount={imageCount}, id={id}, desc={desc}");

                if (provider.FileData.Count == 0)
                {
                    System.Diagnostics.Debug.WriteLine("错误：没有上传文件");
                    return BadRequest("没有上传文件");
                }

                var fileData = provider.FileData[0];
                var originalFileName = fileData.Headers.ContentDisposition.FileName.Trim('"');
                System.Diagnostics.Debug.WriteLine($"原始文件名: {originalFileName}");

                // 创建保存目录
                string uploadDir = ConfigurationManager.AppSettings["UploadSettings:ImageDirectory"] ?? "Uploads";
                string fullPath = "";
                
                try {
                    // 使用固定的C盘路径
                    fullPath = Path.Combine("c:/test", uploadDir);
                    System.Diagnostics.Debug.WriteLine($"尝试使用的保存目录: {fullPath}");
                    
                    // 创建完整路径
                    if (!Directory.Exists("c:/test"))
                    {
                        Directory.CreateDirectory("c:/test");
                        System.Diagnostics.Debug.WriteLine("已创建基础目录 c:/test");
                    }
                    
                    if (!Directory.Exists(fullPath))
                    {
                        Directory.CreateDirectory(fullPath);
                        System.Diagnostics.Debug.WriteLine($"已创建上传目录 {fullPath}");
                    }
                }
                catch (Exception ex)
                {
                    // 如果创建目录失败，尝试使用 Temp 目录
                    System.Diagnostics.Debug.WriteLine($"创建目录失败: {ex.Message}，尝试使用临时目录");
                    fullPath = Path.Combine(Path.GetTempPath(), uploadDir);
                    
                    if (!Directory.Exists(fullPath))
                    {
                        Directory.CreateDirectory(fullPath);
                        System.Diagnostics.Debug.WriteLine($"已创建临时上传目录 {fullPath}");
                    }
                }
                
                System.Diagnostics.Debug.WriteLine($"最终保存目录: {fullPath}");
                
                // 确定文件扩展名
                string fileExtension = Path.GetExtension(originalFileName).ToLowerInvariant();
                if (string.IsNullOrEmpty(fileExtension))
                {
                    // 根据Content-Type确定扩展名
                    if (fileData.Headers.ContentType.MediaType == "application/pdf")
                        fileExtension = ".pdf";
                    else if (fileData.Headers.ContentType.MediaType == "image/tiff")
                        fileExtension = ".tif";
                    else
                        fileExtension = ".dat"; // 默认扩展名
                }

                // 生成唯一文件名
                string fileName = $"{id}_{DateTime.Now:yyyyMMddHHmmss}{fileExtension}";
                string filePath = Path.Combine(fullPath, fileName);

                try
                {
                    // 移动临时文件到目标位置
                    File.Move(fileData.LocalFileName, filePath);
                    System.Diagnostics.Debug.WriteLine($"文件已成功保存到：{filePath}");
                }
                catch (Exception ex)
                {
                    System.Diagnostics.Debug.WriteLine($"移动文件失败：{ex.Message}");
                    // 尝试复制而不是移动
                    try
                    {
                        File.Copy(fileData.LocalFileName, filePath, true);
                        System.Diagnostics.Debug.WriteLine($"文件已通过复制方式保存到：{filePath}");
                    }
                    catch (Exception copyEx)
                    {
                        System.Diagnostics.Debug.WriteLine($"复制文件也失败：{copyEx.Message}");
                        throw;  // 重新抛出异常，让外层catch捕获
                    }
                }
                
                // 记录文件保存路径到日志
                System.Diagnostics.Debug.WriteLine($"文件最终存储位置：{filePath}");

                return Ok(new { 
                    success = true, 
                    msg = "上传成功", 
                    fileName = fileName, 
                    fullPath = filePath  // 返回完整文件路径
                });
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine($"文件上传错误：{ex.Message}");
                System.Diagnostics.Debug.WriteLine($"错误堆栈：{ex.StackTrace}");
                return InternalServerError(ex);
            }
        }
    }
} 